------------------------------------------------------
-- ShoppingList.lua
-- Version 1700.1
------------------------------------------------------

-- Saved configuration & info
FSL_Config = { };
FSL_Config.Enabled = true;

FSL_ShoppingList = { };
-- Has the following internal structure:
--		REALM = {
--			ITEM_LINK,
--		}

-- Constants

function FSL_FinishedAuctionScan()
	FSL_Original_FinishedAuctionScan();
	if (FSL_Config.Enabled) then
		FSL_Report();
	end
end

function FSL_OnLoad()

	-- Register Slash Commands
	SLASH_FSL1 = "/shoppinglist";
	SLASH_FSL2 = "/sl";
	SlashCmdList["FSL"] = function(msg)
		FSL_ChatCommandHandler(msg);
	end
	
	FSL_Original_FinishedAuctionScan = Auctioneer_Event_FinishedAuctionScan;
	Auctioneer_Event_FinishedAuctionScan = FSL_FinishedAuctionScan;
	
	GFWUtils.Print("Fizzwidget ShoppingList initialized!");
	
end

function FSL_ChatCommandHandler(msg)

	-- Print Help
	if ( msg == "help" ) or ( msg == "" ) then
		GFWUtils.Print("Fizzwidget ShoppingList:");
		GFWUtils.Print("/shoppinglist /sl <command>");
		GFWUtils.Print("- "..GFWUtils.Hilite("help").." - Print this helplist.");
		GFWUtils.Print("- "..GFWUtils.Hilite("list").." - Show current list");
		GFWUtils.Print("- "..GFWUtils.Hilite("add <item link>").." - add an item to the list");
		GFWUtils.Print("- "..GFWUtils.Hilite("remove <item link>").." - remove an item from the list");
		if (AUCTIONEER_VERSION ~= nil) then
			GFWUtils.Print("- "..GFWUtils.Hilite("autoreport on").." | "..GFWUtils.Hilite("off").." - Enable or disable printing a report when your AH scans finish.");
			GFWUtils.Print("- "..GFWUtils.Hilite("status").." - Check current settings.");
			GFWUtils.Print("- "..GFWUtils.Hilite("report").." - List \"best deals\" for items on your shopping list in the last auction scan.");
		end
		return;
	end
	
	if (msg == "autoreport on") then
		FSL_Config.Enabled = true;
		if (AUCTIONEER_VERSION ~= nil) then
			GFWUtils.Print("ShoppingList autoreporting is enabled; info will be printed when an AH scan finishes.");
		else
			GFWUtils.Print("ShoppingList autoreporting is enabled but non-functional; Auctioneer is not installed.");
		end
		return;
	end
	if (msg == "autoreport off") then
		FSL_Config.Enabled = false;
		GFWUtils.Print("ShoppingList autoreporting is disabled; no extra info will be printed when an AH scan finishes");
		return;
	end

	-- Check Status
	if ( msg == "status" ) then
		if ( FSL_Config.Enabled ) then
			if (AUCTIONEER_VERSION ~= nil) then
				GFWUtils.Print("ShoppingList autoreporting is enabled; info will be printed when an AH scan finishes.");
			else
				GFWUtils.Print("ShoppingList autoreporting is enabled but non-functional; Auctioneer is not installed.");
			end
		else
			GFWUtils.Print("ShoppingList autoreporting is disabled; no extra info will be printed when an AH scan finishes");
		end
		return;
	end
	
	local realm = GetCVar("realmName");
	local realmList = FSL_ShoppingList[realm];
	
	if ( msg == "list" ) then
		if (realmList == nil or table.getn(realmList) == 0) then
			GFWUtils.Print("Shopping list is empty.");
		else
			GFWUtils.Print("Shopping list:");
			for _, itemLink in realmList do
				GFWUtils.Print(itemLink);
			end
		end
		return;
	end
	
	if ( msg == "report") then
		if (AUCTIONEER_VERSION ~= nil) then
			if (realmList == nil or table.getn(realmList) == 0) then
				GFWUtils.Print("Shopping list is empty.");
			else
				FSL_Report();
			end
		else
			GFWUtils.Print("ShoppingList can't generate a report; Auctioneer is not installed.");
		end
		return;
	end
	
	local _, _, cmd, args = string.find(msg, "(%w+) (.+)");
	for itemLink in string.gfind(args, "|c%x+|Hitem:%d+:%d+:%d+:%d+|h%[.-%]|h|r") do
		if (cmd == "add") then
			if (realmList ~= nil and GFWTable.IndexOf(realmList, itemLink) ~= 0) then
				GFWUtils.Print(itemLink.." already in shopping list.");
			else
				if (realmList == nil) then
					FSL_ShoppingList[realm] = {};
					realmList = FSL_ShoppingList[realm];
				end
				table.insert(realmList, itemLink);
				GFWUtils.Print("Added "..itemLink.." to shopping list.");
			end
		elseif (cmd == "remove") then
			if (realmList == nil or table.getn(realmList) == 0) then
				GFWUtils.Print("Shopping list is empty.");
			else
				local index = GFWTable.IndexOf(realmList, itemLink);
				if (index > 0) then
					table.remove(realmList, index);
					GFWUtils.Print("Removed "..itemLink.." from shopping list.");
				else
					GFWUtils.Print("Could not find "..itemLink.." in shopping list.");
				end
			end
		end
	end	
	
end

function FSL_IsItemWithBuyoutFilter(itemLink, signature)
	local id, rprop, enchant, name, count, min, buyout, uniq = Auctioneer_GetItemSignature(signature);
	if (buyout > 0 and string.find(itemLink, id) and string.find(itemLink, name)) then
		return false;
	else
		return true;
	end
end

function FSL_Report()
	local realm = GetCVar("realmName");
	local realmList = FSL_ShoppingList[realm];
	
	if realmList == nil then return; end
	
	for _, itemLink in realmList do
		
		-- get all the auctions for this item
		local itemAuctions = Auctioneer_QuerySnapshot(FSL_IsItemWithBuyoutFilter, itemLink);
		
		if (table.getn(itemAuctions) > 0) then
		
			-- sort based on amount below median price
			table.sort(itemAuctions, FSL_GoodDealSort);

			-- show the top auction for each item
			local id, rprop, enchant, name, count, min, buyout, uniq = Auctioneer_GetItemSignature(itemAuctions[1].signature);
			local countString = "";
			if (count > 1) then
				countString = GFWUtils.Hilite(count.."x");
			end
			local percent = FSL_PercentBuyoutBelowMedian(itemAuctions[1].signature);
			local percentSummary;
			if (percent > 0) then
				percentSummary = percent.."% below median buyout";
			elseif (percent < 0) then
				percentSummary = percent.."% "..GFWUtils.Red("above").." median buyout";
			else
				percentSummary = "matches median buyout";
			end
			GFWUtils.Print(countString..itemLink.." available for "..TT_GetTextGSC(buyout)..GFWUtils.Hilite(" ("..percentSummary..")")..".");
		end
	end
end

function FSL_PercentBuyoutBelowMedian(itemSignature)
	local id, rprop, enchant, name, count, min, buyout, uniq = Auctioneer_GetItemSignature(itemSignature);
	local itemKey = id .. ":" .. rprop..":"..enchant;
	local medBuyout = Auctioneer_GetUsableMedian(itemKey, name);
	if (medBuyout == nil) then
		medBuyout = Auctioneer_GetItemHistoricalMedianBuyout(itemKey, name);
	end
	return Auctioneer_PercentLessThan(medBuyout * count, buyout);
end

-- method to pass to table.sort() that sorts auctions by amount less than median price
function FSL_GoodDealSort(a, b)	
	return (FSL_PercentBuyoutBelowMedian(a.signature) > FSL_PercentBuyoutBelowMedian(b.signature)) 
end

